@@ -4,7 +4,7 @@ This is the base multidimensional array class which implements all
 structural array operations but treats the array contents as opaque
 objects
 
-$Id: generic.py,v 1.74 2005/04/06 19:24:31 jaytmiller Exp $
+$Id: generic.py,v 1.83 2006/01/16 15:16:37 jaytmiller Exp $
 """
 
 import types as _types
@@ -18,15 +18,14 @@ import numinclude
 import copy
 import copy_reg
 
-from _ndarray import _ndarray
-from _ndarray import _isIntegerSequence
-from _ndarray import product
-from _ndarray import _alignment
+from _ndarray import _ndarray, _isIntegerSequence, product, _alignment, ravel
+
 import _ufunc
 
 _PROTOTYPE = 0  # Set to 1 to activate Python prototypes of C code.
 
 _UPDATEDICT = 0x1000  # views update from dict of original
+_WRITABLE = 0x400 # array is WRITABLE
 
 def _buffer_reduce(b):
     """Converts a "buffer" object to a pickling reduction tuple."""
@@ -205,6 +204,23 @@ class NDArray(_ndarray):
     Subclasses must provide the semantical interpretation of elements,
     including the __str__, __repr__, _getitem, and _setitem methods.
 
+
+    NDArray(shape=None, itemsize=0, buffer=None, byteoffset=0, bytestride=0, aligned=1)
+
+    shape               The shape of the array.
+
+    itemsize            The size in bytes of a single element.
+
+    buffer              An object meeting the Python buffer protocol which will store the
+                        array data.  If None is specified,  a memory oject is created.
+
+    byteoffset          The offset in bytes from the base of buffer to the array data.
+
+    bytestride          The distance in bytes between elements.  Defaults to itemsize.
+
+    aligned             A flag describing whether buffer+offset is aligned for itemsize.
+
+
     Given an array index of arr[k,j,i] it is always true that the
     byte offset of the element in the array is computed thusly:
 
@@ -488,24 +504,13 @@ class NDArray(_ndarray):
                              self._strides[axis1+1:axis2] +
                              (self._strides[axis1],) + self._strides[axis2+1:])
 
+        def ravel(self):
+            """ravel(self) setshapes 'self' into an equivalent 1D array.
+            """
+            self.setshape((self.nelements(),))
+
     size = _ndarray.nelements  # more portable synonym
 
-    def __get_array_shape__(self):
-        return self._shape
-
-    __array_shape__ = property(__get_array_shape__, None, "")
-
-    def __get_array_data__(self):
-        return self._data
-
-    __array_data__ = property(__get_array_data__, None, "")
-
-    def __get_array_strides__(self):
-        return self._strides
-
-    __array_strides__ = property(__get_array_strides__, None, "")
-    
-
     def __getstate__(self):
         """returns state of NDArray for pickling."""
         state = copy.copy(self.__dict__)
@@ -677,7 +682,7 @@ class NDArray(_ndarray):
     def setshape(self, shape, *args):
         """Change array shape in place.  Call as setshape(i,j,k) or setshape((i,j,k))."""
         if not self.iscontiguous():
-            raise TypeError("Can't reshape non-contiguous numarray")
+            raise ValueError("Can't reshape non-contiguous numarray")
         shape = list(getShape(shape, *args))
         # look for index = -1, which indicates an expandable dimension
         nelements = self.nelements()
@@ -714,7 +719,7 @@ class NDArray(_ndarray):
         a = self.view()
         a.ravel()
         a[:] = flat
-
+            
     flat = property(getflat, setflat, doc="access to array as 1D")
 
     def copy(self):
@@ -742,6 +747,9 @@ class NDArray(_ndarray):
         Copies are always contiguous, but no conversions are implied
 
         """
+        if self.rank == 0:
+            self = self.view()
+            self.shape = (1,)
         return _bytes.copyToString(self._shape, self._data, self._byteoffset,
                                  self._strides, self._itemsize)
 
@@ -916,11 +924,6 @@ class NDArray(_ndarray):
         """
         return repeat(self, repeats, axis)
 
-    def ravel(self):
-        """ravel(self) setshapes 'self' into an equivalent 1D array.
-        """
-        self.setshape((self.nelements(),))
-
     def factory(self, *args, **keys):
         """factory(...) calls the array(...) factory function defined in the
         source module where the class of 'self' was defined.
@@ -963,10 +966,11 @@ def reshape(arr, shape, *args):
     v.setshape(shape, *args)
     return v
 
-def ravel(array):
-    """Returns a *view* of array reshaped as 1D."""
-    array = _nc.asarray(array)
-    return reshape(array, (array.nelements(),))
+if _PROTOTYPE:
+    def ravel(array):
+        """Returns a *view* of array reshaped as 1D."""
+        array = _nc.asarray(array)
+        return reshape(array, (array.nelements(),))
 
 def fromstring(datastring): pass
 
@@ -1167,13 +1171,13 @@ def indices(shape, type=None):
         a = a.astype(type)
     return a
 
-def fromfunction(function, dimensions): # from Numeric
+def fromfunction(function, dimensions, type=None): # from Numeric
     """fromfunction() returns an array constructed by calling function
     on a tuple of number grids.  The function should accept as many
     arguments as there are dimensions which is a list of numbers
     indicating the length of the desired output for each axis.
     """
-    return apply(function, tuple(indices(dimensions)))
+    return apply(function, tuple(indices(dimensions, type)))
 
 def _broadcast_or_resize(a, b):
     try:
@@ -1197,4 +1201,3 @@ def putmask(array, mask, values):
         bmask = _broadcast_or_resize(array, bmask)
     bvalues = _broadcast_or_resize(array, bvalues)
     choose(bmask != 0, (array, bvalues), array)
-
