@@ -25,6 +25,9 @@ staticforward PyTypeObject _ndarray_type;
 static PyObject *zero, *one, *NewAxisSlice, *FullSlice, *NullArgs, *NullKeywds;
 
 static PyObject *
+_x_view(PyArrayObject *self);
+
+static PyArrayObject *
 _view(PyArrayObject *self);
 
 static PyObject *
@@ -135,6 +138,18 @@ _ndarray_fragile_data_get(PyArrayObject *self)
 	return PyLong_FromUnsignedLong((unsigned long) self->data);
 }
 
+static PyObject *
+_ndarray_dataptr_get(PyArrayObject *self)
+{
+	char address[40], *ap;
+	snprintf(address, sizeof(address), "%p", self->data);
+	ap = address;
+	if (strncmp(address, "0x", 2) == 0) {
+		ap += 2;
+	}	
+	return Py_BuildValue("(s,i)", ap, (self->flags & WRITABLE) == 0);
+}
+
 static int
 _ndarray_data_set(PyArrayObject *self, PyObject *s)
 {
@@ -245,14 +260,25 @@ static PyGetSetDef _ndarray_getsets[] = {
 	 (getter)_ndarray_shape_get, 
 	 (setter)_ndarray_shape_set, 
 	 "tuple of array dimensions"}, 
+	{"__array_shape__", /* scipy newcore */
+	 (getter)_ndarray_shape_get, 
+	 (setter)_ndarray_shape_set, 
+	 "tuple of array dimensions (scipy newcore array interface)"}, 
 	{"_strides", 
 	 (getter)_ndarray_strides_get, 
 	 (setter)_ndarray_strides_set, 
 	 "tuple of byte steps"}, 
+	{"__array_strides__",  /* scipy newcore */
+	 (getter)_ndarray_strides_get, 
+	 (setter)_ndarray_strides_set, 
+	 "tuple of byte steps (scipy newcore array interface)"}, 
 	{"_data", 
 	 (getter)_ndarray_data_get,
 	 (setter)_ndarray_data_set, 
 	 "buffer object holding array data"}, 
+	{"__array_data__", 
+	 (getter)_ndarray_dataptr_get, 0,
+	 "(dataptr, readonly) (scipy newcore array interface)"}, 
 	{"_fragile_data", 
 	 (getter)_ndarray_fragile_data_get,
 	 (setter)NULL, 
@@ -417,6 +443,8 @@ _ndarray_dealloc(PyObject *self)
 		if (have_error)
 			PyErr_Restore(err_type, err_value, err_traceback);
 	}
+	Py_XDECREF(me->base);
+	me->base = NULL;
 	Py_XDECREF(me->_data);
 	me->_data = NULL;
 	self->ob_type->tp_free(self);
@@ -512,11 +540,8 @@ _simpleIndexingCore(PyArrayObject *self, long offset, int nindices, PyObject *va
 		}
 	} else {  /* partially subscripted --> subarray */
 		long i;
-		if (NA_NumArrayCheck((PyObject *)self))		      
-			result = (PyArrayObject *) _view(self);
-		else
-			result = (PyArrayObject *) PyObject_CallMethod(
-				(PyObject *) self,"view",NULL);
+
+		result = _view(self);
 		if (!result) goto _exit;
 		
 		result->nd = result->nstrides = self->nd - nindices;
@@ -1180,42 +1205,17 @@ _ndarray_iscontiguous(PyArrayObject *self, PyObject *args)
 	return PyInt_FromLong((self->flags & CONTIGUOUS) != 0);
 }
 
-/* Added for f2py support.  Not used internally. */
-static int 
-is_fortran_contiguous(const PyArrayObject *ap) 
-{
-	/* array_has_column_major_storage(a) is equivalent to
-	   transpose(a).iscontiguous() but more efficient.
-	   
-	   This function can be used in order to decide whether to use a
-	   Fortran or C version of a wrapped function. This is relevant, for
-	   example, in choosing a clapack or flapack function depending on
-	   the storage order of array arguments.
-
-	   This function was taken (with a small modification for broadcast numarray)
-	   from f2py.
-	*/
-	int sd;
-	int i;
-	sd = ap->descr->elsize;
-	for (i=0;i<ap->nd;++i) {
-		if (ap->dimensions[i] == 0) return 0;  /* broadcast array */
-		if (ap->strides[i] != sd) return 0;
-		sd *= ap->dimensions[i];
-	}
-	return 1;
-}
-
 static PyObject *
 _ndarray_is_fortran_contiguous(PyArrayObject *self, PyObject *args)
 {
 	if (!PyArg_ParseTuple(args, ":is_fortran_contiguous"))
 		return NULL;
-	return PyInt_FromLong(is_fortran_contiguous(self));
+	NA_updateContiguous(self);
+	return PyInt_FromLong((self->flags & FORTRAN_CONTIGUOUS) != 0);
 }
 
 static PyObject *
-_view(PyArrayObject *self)
+_x_view(PyArrayObject *self)
 {
 	PyArrayObject *view=NULL;
 	PyObject *vd=NULL, *sd=NULL;
@@ -1268,12 +1268,23 @@ _view(PyArrayObject *self)
 	return (PyObject *) view;
 }
 
+static PyArrayObject *
+_view(PyArrayObject *self)
+{
+	PyObject *result;
+	if (NA_NumArrayCheck((PyObject *)self))		      
+		result = _x_view(self);
+	else
+		result = PyObject_CallMethod((PyObject *) self,"view",NULL);
+	return (PyArrayObject *) result;
+}
+
 static PyObject *
 _ndarray_view(PyArrayObject *self, PyObject *args, PyObject *keywds)
 {
 	if (!PyArg_ParseTuple(args, ":view"))
 		return NULL;
-	return _view(self);
+	return _x_view(self);
 }
 
 static PyObject *
@@ -1332,6 +1343,33 @@ _ndarray_swapaxes(PyArrayObject *self, PyObject *args)
 	return Py_None;
 }
 
+static PyObject *
+_ravel(PyObject *self)
+{
+	PyArrayObject *me = (PyArrayObject *) self;
+	NA_updateStatus(me);
+	if (PyArray_ISCONTIGUOUS(me)) {
+		me->dimensions[0] = NA_elements(me);
+		me->strides[0] = me->bytestride;
+		me->nstrides = me->nd = 1;
+		return (PyObject *) me;
+	} else {
+		return PyErr_Format(PyExc_ValueError,
+				    "Can't reshape non-contiguous numarray.");
+	}
+}
+
+static PyObject *
+_ndarray_ravel(PyObject *self, PyObject *args)
+{
+	if (!PyArg_ParseTuple(args, ":ravel"))
+		return NULL;
+	if (!_ravel(self))
+		return NULL;
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
 /*  
 
 See generic._takeShape to see where result.shape comes from and how it is
@@ -1473,6 +1511,8 @@ _ndarray_methods[] = {
 	{"_simpleIndexing", (PyCFunction)_ndarray_simpleIndexing, 
 	 METH_VARARGS | METH_KEYWORDS,
 	 "_simpleIndexing(key, value=None) gets or sets for integer sequence key"},
+	{"ravel", (PyCFunction) _ndarray_ravel, METH_VARARGS,
+	 "ravel() ->  1D view of self."},
 	{"tolist", tolist, METH_VARARGS,
 	 "tolist() ->  array converted to nested list of scalars."},
 	{"nelements", (PyCFunction)_ndarray_nelements, METH_VARARGS,
@@ -1489,6 +1529,25 @@ _ndarray_methods[] = {
 	{NULL,	NULL},
 };
 
+static char _ndarray_tpdoc[] = 
+"_ndarray is a private C baseclass used in the implementation of NDArray.\n"
+"\n"
+"_ndarray(shape=None, itemsize=0, buffer=None, byteoffset=0, bytestride=0, aligned=1)\n"
+"\n"
+"shape               The shape of the array.\n"
+"\n"
+"itemsize            The size in bytes of a single element.\n"
+"\n"
+"buffer              An object meeting the Python buffer protocol which will\n"
+"                    store the array data.  If None is specified,  a memory \n"
+"                    oject is created.\n"
+"\n"
+"byteoffset          The offset in bytes from the base of buffer to the array data.\n"
+"\n"
+"bytestride          The distance in bytes between elements.  Defaults to itemsize.\n"
+"\n"
+"aligned             A flag describing whether buffer+offset is aligned for itemsize.\n";
+
 static PyTypeObject 
 _ndarray_type = {
 	PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
@@ -1517,7 +1576,7 @@ _ndarray_type = {
 	&array_as_buffer,			/* tp_as_buffer */
 	Py_TPFLAGS_DEFAULT | 
         Py_TPFLAGS_BASETYPE,                    /* tp_flags */
-	0,					/* tp_doc */
+	_ndarray_tpdoc, 			/* tp_doc */
 	0,					/* tp_traverse */
 	0,					/* tp_clear */
 	0,					/* tp_richcompare */
@@ -1588,26 +1647,43 @@ _ndarray_is_buffer(PyObject *module, PyObject *args)
 	return PyInt_FromLong(ans);
 }
 
+PyObject *
+_ndarray_ravel_function(PyObject *module, PyObject *args)
+{
+	PyObject *o, *b;
+	PyArrayObject *a;
+	if (!PyArg_ParseTuple(args, "O:ravel", &o))
+		return NULL;
+	if (NA_NDArrayCheck(o)) {
+		a = _view((PyArrayObject *)o);
+	} else {
+		a = NA_InputArray(o, tAny, 0);
+	}
+	if (!a) return NULL;
+	NA_updateStatus(a);
+	if (!PyArray_ISCONTIGUOUS(a)) {
+		b = (PyObject *) NA_copy(a);
+		if (!b) return NULL;
+		Py_DECREF(a);
+		a = (PyArrayObject *) b;
+	}
+	if (!_ravel((PyObject *) a)) return NULL;
+	return (PyObject *) a;
+}
+
 static PyMethodDef 
 _ndarray_functions[] = {
-	{"_isIntegerSequence", 
-	 (PyCFunction) _ndarray_isIntegerSequence, 
-	 METH_VARARGS,
+	{"_isIntegerSequence", (PyCFunction) _ndarray_isIntegerSequence, METH_VARARGS,
 	 "_isIntegerSequence(s) returns 1 if all elements of sequence s are integers, else 0"},
-	{"product", 
-	 (PyCFunction) _ndarray_product,
-	 METH_VARARGS,
+	{"ravel", (PyCFunction) _ndarray_ravel_function, METH_VARARGS,
+	 "ravel(x) converts x to an array and either returns a flattened view or copy as needed."},
+	{"product", (PyCFunction) _ndarray_product, METH_VARARGS,
 	 "product(t)   returns the product of all elements of an integer sequence"},
-	{"lp64", 
-	 (PyCFunction) _ndarray_lp64,
-	 METH_VARARGS,
+	{"lp64", (PyCFunction) _ndarray_lp64, METH_VARARGS,
 	 "lp64()  returns flag indicating whether this system and compiler use the 64-bit LP64 model."},
-	{"hasUInt64", 
-	 (PyCFunction) _ndarray_hasUInt64,
-	 METH_VARARGS,
+	{"hasUInt64", (PyCFunction) _ndarray_hasUInt64, METH_VARARGS,
 	 "hasUInt64()  returns flag indicating whether this system has 64-bit unsigned integer support."},
-	{"is_buffer", (PyCFunction) _ndarray_is_buffer,
-	 METH_VARARGS,
+	{"is_buffer", (PyCFunction) _ndarray_is_buffer, METH_VARARGS,
 	 "is_buffer(o)  returns 1 iff 'o' should work as a buffer."},
 	{NULL,		NULL}		/* sentinel */
 };
@@ -1627,7 +1703,6 @@ init_ndarray(void)
 {
 	PyObject *m;
 
-	_ndarray_type.tp_base = NULL;
 	_ndarray_type.tp_alloc = PyType_GenericAlloc;
 
 	if (PyType_Ready(&_ndarray_type) < 0)
