@@ -9,10 +9,10 @@
 #include "tc.h"
 
 char *_ufunc__doc__ =
-"_ufunc is a module which supplies the _ufunc baseclass of Ufunc.  Class\n"
-"_ufunc is a baseclass used to accelerate selected methods of Ufunc by\n"
-"handling simple cases directly in C.  More complex cases are delegated to\n"
-"python coded methods.\n";
+	"_ufunc is a module which supplies the _ufunc baseclass of Ufunc.  Class\n"
+	"_ufunc is a baseclass used to accelerate selected methods of Ufunc by\n"
+	"handling simple cases directly in C.  More complex cases are delegated to\n"
+	"python coded methods.\n";
 
 /* ====================================================================== */
 
@@ -37,14 +37,14 @@ static PyObject *pUnknownOperator;
 #define _PYTHON_CALLBACKS 0
 
 #if _PYTHON_CALLBACKS
-     #define ConverterRebuffer(conv,arr,inb) \
-              PyObject_CallMethod(conv, "rebuffer", "(OO)", arr, inb)
+#define ConverterRebuffer(conv,arr,inb)				\
+	PyObject_CallMethod(conv, "rebuffer", "(OO)", arr, inb)
 #else
-     #define ConverterRebuffer(conv,arr,inb) \
+#define ConverterRebuffer(conv,arr,inb)				\
 	((PyConverterObject *)conv)->rebuffer(conv, arr, inb)
 #endif
 
-#define ConverterClean(conv, arr) \
+#define ConverterClean(conv, arr)			\
         ((PyConverterObject *)conv)->clean(conv, arr)
 
 /* ====================================================================== */
@@ -64,6 +64,12 @@ _copyFromAndConvert(PyObject *from, PyObject *to);
 
 static int deferred_ufunc_init(void);
 
+static PyObject *
+_cached_dispatch1(PyObject *self, PyObject *in1, PyObject *out);
+
+static PyObject *
+_cached_dispatch2(PyObject *self, PyObject *in1, PyObject *in2, PyObject *out);
+
 /* ====================================================================== */
 
 static PyObject * 
@@ -133,69 +139,69 @@ _normalize_results(int nin,     PyObject **inputs,
 static int
 _tuple_check(PyObject *o, char *name)
 {
-    if (PyTuple_Check(o)) return 0;
+	if (PyTuple_Check(o)) return 0;
 
-    PyErr_Format
-      (PyExc_TypeError, "%s should be a tuple", name);
+	PyErr_Format
+		(PyExc_TypeError, "%s should be a tuple", name);
 
-    return -1;
+	return -1;
 }
 
 static PyObject * 
 _Py_normalize_results(PyObject *module, PyObject *args)
 {
-    PyObject *tinputs, *toutputs, *tresults;
-    PyObject *inputs[MAXARRAYS], *outputs[MAXARRAYS], *results[MAXARRAYS];
-    int i, return_rank1 = 0;
-    int nin, nout, nresult;
+	PyObject *tinputs, *toutputs, *tresults;
+	PyObject *inputs[MAXARRAYS], *outputs[MAXARRAYS], *results[MAXARRAYS];
+	int i, return_rank1 = 0;
+	int nin, nout, nresult;
 
-    if (!PyArg_ParseTuple(args, "OOO|i:_normalize_results", 
-			  &tinputs, &toutputs, &tresults, &return_rank1))
-	return NULL;
+	if (!PyArg_ParseTuple(args, "OOO|i:_normalize_results", 
+			      &tinputs, &toutputs, &tresults, &return_rank1))
+		return NULL;
 
-    if (_tuple_check(tinputs, "inputs") < 0)
-	return NULL;
+	if (_tuple_check(tinputs, "inputs") < 0)
+		return NULL;
 
-    nin = PyTuple_Size(tinputs);
-    for(i=0; i<nin; i++) {
-	inputs[i] = PyTuple_GetItem(tinputs, i);
-	if (!inputs[i]) return NULL;
-    }
+	nin = PyTuple_Size(tinputs);
+	for(i=0; i<nin; i++) {
+		inputs[i] = PyTuple_GetItem(tinputs, i);
+		if (!inputs[i]) return NULL;
+	}
     
-    if (toutputs == Py_None) {
-	nout = 0;
-    } else if (_tuple_check(toutputs, "outputs") < 0) {
-	return NULL;
-    } else {
-	nout = PyTuple_Size(toutputs);
-	for(i=0; i<nout; i++) {
-	    outputs[i] = PyTuple_GetItem(toutputs, i);
-	    if (!outputs[i]) return NULL;
+	if (toutputs == Py_None) {
+		nout = 0;
+	} else if (_tuple_check(toutputs, "outputs") < 0) {
+		return NULL;
+	} else {
+		nout = PyTuple_Size(toutputs);
+		for(i=0; i<nout; i++) {
+			outputs[i] = PyTuple_GetItem(toutputs, i);
+			if (!outputs[i]) return NULL;
+		}
 	}
-    }
 
-    if (_tuple_check(tresults, "results") < 0)
-	return NULL;
+	if (_tuple_check(tresults, "results") < 0)
+		return NULL;
 
-    nresult = PyTuple_Size(tresults);
-    for(i=0; i<nresult; i++) {
-	results[i] = PyTuple_GetItem(tresults, i);
-	if (!results[i]) return NULL;
-    }
+	nresult = PyTuple_Size(tresults);
+	for(i=0; i<nresult; i++) {
+		results[i] = PyTuple_GetItem(tresults, i);
+		if (!results[i]) return NULL;
+	}
 
-    if ((return_rank1 != 0) && (return_rank1 != 1))
-	return PyErr_Format(
-	    PyExc_ValueError, "return_rank1 should be 0 or 1.");
+	if ((return_rank1 != 0) && (return_rank1 != 1))
+		return PyErr_Format(
+			PyExc_ValueError, "return_rank1 should be 0 or 1.");
 
-    return _normalize_results(
-	nin, inputs, nout, outputs, nresult, results, return_rank1);
+	return _normalize_results(
+		nin, inputs, nout, outputs, nresult, results, return_rank1);
 }
 
 /* _firstcol is an array mutator that changes an array into
-a view of its first column.  _firstcol_undo undoes the mutation.
-This approach is an optimization over the Python prototype's clean
-but slow view based approach.  This ugliness avoids 1-2 view method
-callbacks which are fairly expensive.
+   a view of its first column.  _firstcol_undo undoes the mutation.
+   This approach is an optimization over the Python prototype's clean
+   but slow view based approach.  This ugliness avoids 1-2 view method
+   callbacks which are fairly expensive.
 */
 typedef struct 
 {
@@ -229,7 +235,7 @@ _firstcol_undo(PyObject *arr, firstcol_undo *undo)
 
 static PyObject *
 CheckFPErrors(PyObject *self, PyObject *args) {
-    return Py_BuildValue("i", NA_checkFPErrors());
+	return Py_BuildValue("i", NA_checkFPErrors());
 }
 
 
@@ -243,7 +249,7 @@ _getNewArray(PyObject *master, PyObject *type)
 
 	if (NA_NumArrayCheckExact(master)) {
 		out = (PyObject *) NA_vNewArray( NULL, typenum, 
-				    amaster->nd, amaster->dimensions);
+						 amaster->nd, amaster->dimensions);
 	} else {
 		out = PyObject_CallMethod(master, "new", "(O)", type);
 	}
@@ -334,28 +340,7 @@ _Py_digest(PyObject *module, PyObject *args)
 	}
 }
 
-staticforward PyTypeObject _ufunc_type;
-
-typedef struct
-{
-	long in1, in2, out, thread_id;
-	char cumop[12]; 
-	PyObject *type;
-	PyObject *ctuple;
-} cache_entry;
-
-typedef struct
-{
-	int insert;
-	cache_entry entry[20];
-} ufunc_cache;
-
-typedef struct {
-	PyObject_HEAD
-	PyObject *operator;
-	int    arity;
-	ufunc_cache cache;
-} PyUfuncObject;
+/* staticforward PyTypeObject _ufunc_type;   XXX delete me */
 
 static int
 _reportErrors(PyObject *ufunc, PyObject *result, PyObject *out)
@@ -365,7 +350,7 @@ _reportErrors(PyObject *ufunc, PyObject *result, PyObject *out)
 		return -1;
 	if (!result) return -1;
 	Py_DECREF(result);  /* Py_None */
-	return NA_checkAndReportFPErrors(PyString_AS_STRING(self->operator));
+	return NA_checkAndReportFPErrors(PyString_AS_STRING(self->oprator));
 }
 
 #if defined(UNTHREADED)
@@ -373,11 +358,11 @@ static long PyThread_get_thread_ident() { return 0; }
 #endif
  
 static void 
-_cache_insert(ufunc_cache *cache, PyObject *ctuple,
+_cache_insert(_ufunc_cache *cache, PyObject *ctuple,
 	      long in1, long in2, long out, 
-	     char *cumop, PyObject *type)
+	      char *cumop, PyObject *type)
 {
-	cache_entry *entry = &cache->entry[cache->insert];
+	_cache_entry *entry = &cache->entry[cache->insert];
 	++ cache->insert;
 	cache->insert %= ELEM(cache->entry);
 	Py_XDECREF(entry->type);
@@ -414,13 +399,13 @@ _Py_cache_insert(PyObject *self, PyObject *args)
 }
 
 static void 
-_cache_flush(ufunc_cache *cache)
+_cache_flush(_ufunc_cache *cache)
 {
 	int i;
 	for(i=0; i<ELEM(cache->entry); i++) {
 		Py_XDECREF(cache->entry[i].type);
 		Py_XDECREF(cache->entry[i].ctuple);
-		memset(&cache->entry[i], 0, sizeof(cache_entry));
+		memset(&cache->entry[i], 0, sizeof(_cache_entry));
 	}
 }
 
@@ -437,7 +422,7 @@ _Py_cache_flush(PyObject *self, PyObject *args)
 
 static PyObject *
 _cache_lookup(
-	ufunc_cache *cache, PyObject *in1, PyObject *in2, PyObject *out, 
+	_ufunc_cache *cache, PyObject *in1, PyObject *in2, PyObject *out, 
 	char *cumop, PyObject  *type)
 {
 	long  i;
@@ -447,7 +432,7 @@ _cache_lookup(
 	long thread_id = PyThread_get_thread_ident();
 
 	for(i=0; i<ELEM(cache->entry); i++) {
-		cache_entry *c = &cache->entry[i];
+		_cache_entry *c = &cache->entry[i];
 		if ((din1 == c->in1) &&
 		    (din2 == c->in2) &&
 		    (dout == c->out) &&
@@ -480,69 +465,110 @@ _Py_cache_lookup(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-_ufunc_arity_get(PyUfuncObject *self)
+_cached_dispatch(PyUfuncObject *self, 
+		 int n_ins, PyObject *ins[], 
+		 int n_outs, PyObject *outs[])
 {
-	return PyInt_FromLong(self->arity);
+	if ((self->n_inputs == 1) && (self->n_outputs==1)) {
+		if ((n_ins != 1) || (n_outs != 1))
+			return PyErr_Format(
+				PyExc_RuntimeError,
+				"wrong number of parameters to unary ufunc.");
+		return _cached_dispatch1(
+			(PyObject *) self, ins[0], outs[0]);
+	} else if ((self->n_inputs == 2) && (self->n_outputs==1)) {
+		if ((n_ins != 2) || (n_outs != 1))
+			return PyErr_Format(
+				PyExc_RuntimeError,
+				"wrong number of parameters to binary ufunc.");
+		return _cached_dispatch2(
+			(PyObject*) self, ins[0], ins[1], outs[0]);
+	} else
+		return PyErr_Format(
+			PyExc_RuntimeError, 
+			"N-ary ufunc C interface is not implemented yet.");
+	  
 }
 
-static int
-_ufunc_arity_set(PyUfuncObject *self, PyObject *s)
+static PyObject *
+_ufunc_new(PyTypeObject *type, PyObject  *args, PyObject *kwds)
 {
-	if (!s) { 
-		PyErr_Format(PyExc_RuntimeError, "can't delete _arity"); 
-		return -1; 
-	}
-	if (!PyInt_Check(s)) {
-		PyErr_Format(PyExc_TypeError, 
-			     "_ufunc_arity_set:  arity must be an int.");
-		return -1;
-	}
-	self->arity = PyInt_AsLong(s);
-	if (self->arity < 0 || self->arity > 2) {
-		PyErr_Format(PyExc_ValueError,
-			     "_ufunc_arity_set:  arity out of range 0..2");
-		self->arity = 0;
-		return -1;
-	}
-	return 0;
+	PyUfuncObject *self;
+	PyObject *ufuncs;
+	if (deferred_ufunc_init() < 0) return NULL;
+
+	self = (PyUfuncObject *) PyType_GenericNew(type, args, kwds);
+	if (!self) return NULL;
+	
+	if (!PyArg_ParseTuple(args, "OOiiO", 
+			      &self->oprator, &ufuncs, 
+			      &self->n_inputs, &self->n_outputs,
+			      &self->identity))
+		return NULL;
+	
+	memset(&self->cache, 0, sizeof(self->cache));
+	Py_INCREF(self->oprator);
+	Py_INCREF(self->identity);
+	
+	self->call = (_ufunc_function) _cached_dispatch;
+
+	return (PyObject *) self;
+}
+
+static void
+_ufunc_dealloc(PyObject *self)
+{
+	PyUfuncObject *me = (PyUfuncObject*) self;
+	_cache_flush(&me->cache);
+	Py_DECREF(me->oprator);
+	Py_DECREF(me->identity);
+	self->ob_type->tp_free(self);
+}
+
+static PyObject *
+_ufunc_n_inputs_get(PyUfuncObject *self)
+{
+	return PyInt_FromLong(self->n_inputs);
+}
+
+static PyObject *
+_ufunc_n_outputs_get(PyUfuncObject *self)
+{
+	return PyInt_FromLong(self->n_outputs);
 }
 
 static PyObject *
 _ufunc_operator_get(PyUfuncObject *self)
 {
-	Py_INCREF(self->operator);
-	return self->operator;
+	Py_INCREF(self->oprator);
+	return self->oprator;
 }
 
-static int
-_ufunc_operator_set(PyUfuncObject *self, PyObject *s)
+static PyObject *
+_ufunc_identity_get(PyUfuncObject *self)
 {
-	if (!s) { 
-		PyErr_Format(PyExc_RuntimeError, 
-			     "_ufunc_operator_set:  operator can't be deleted.");
-		return -1;
-	}
-	if (!PyString_Check(s)) {
-		PyErr_Format(PyExc_TypeError, 
-			     "_ufunc_operator_set:  operator must be a string.");
-		return -1;
-	}
-	Py_INCREF(s);
-	Py_DECREF(self->operator);
-	self->operator = s;
-	return 0;
+	Py_INCREF(self->identity);
+	return self->identity;
 }
 
 static PyGetSetDef 
 _ufunc_getsets[] = 
 {
-	{"arity", 
-	 (getter)_ufunc_arity_get, 
-	 (setter)_ufunc_arity_set, 
-	 "determines if Ufunc is unary or binary"},
+	{"n_inputs", 
+	 (getter)_ufunc_n_inputs_get, 
+	 (setter) NULL,
+	 "specifies ufunc inputs count)"},
+	{"n_outputs", 
+	 (getter)_ufunc_n_outputs_get, 
+	 (setter) NULL,
+	 "specifies ufunc outputs count)"},
+	{"_identity", 
+	 (getter)_ufunc_identity_get, 
+	 (setter) NULL,
+	 "specifies ufunc's identity value"},
 	{"operator", 
 	 (getter)_ufunc_operator_get, 
-	 (setter)_ufunc_operator_set, 
+	 (setter)NULL,
 	 "name of the ufunc operator"},
 	{0}
 };
@@ -615,55 +641,24 @@ _Py_getBufferSize(PyObject *module, PyObject *args)
 	return PyInt_FromLong(buffersize);
 }
 
-static PyObject *
-_ufunc_new(PyTypeObject *type, PyObject  *args, PyObject *kwds)
-{
-	PyUfuncObject *self;
-	
-	if (deferred_ufunc_init() < 0) return NULL;
-
-	self = (PyUfuncObject *) PyType_GenericNew(type, args, kwds);
-	if (!self) return NULL;
-	
-	memset(&self->cache, 0, sizeof(self->cache));
-	self->operator = pUnknownOperator;
-	Py_INCREF(pUnknownOperator);
-	return (PyObject *) self;
-}
-
-static void
-_ufunc_dealloc(PyObject *self)
-{
-	PyUfuncObject *me = (PyUfuncObject*) self;
-	_cache_flush(&me->cache);
-	Py_DECREF(me->operator);
-	self->ob_type->tp_free(self);
-}
-
-static PyObject *
-_cached_dispatch1(PyObject *self, PyObject *in1, PyObject *out);
-
-static PyObject *
-_cached_dispatch2(PyObject *self, PyObject *in1, PyObject *in2, PyObject *out);
-
 static PyObject *
 _ufunc_call( PyUfuncObject *self, PyObject *args)
 {
 	PyObject *in1, *in2 = Py_None, *out = Py_None;
 	char spec[200];
-	switch(self->arity) 
+	switch(self->n_inputs) 
 	{
 	case 0: default:
 		return PyErr_Format(
 			PyExc_RuntimeError, 
 			"_ufunc_call: __call__ is not implemented by base UFunc class");
 	case 1:
-		snprintf(spec, sizeof(spec), "O|O:%s", PyString_AS_STRING(self->operator));
+		snprintf(spec, sizeof(spec), "O|O:%s", PyString_AS_STRING(self->oprator));
 		if (!PyArg_ParseTuple(args, spec, &in1, &out))
 			return NULL;
 		return _cached_dispatch1((PyObject *) self, in1, out); 
 	case 2:
-		snprintf(spec, sizeof(spec), "OO|O:%s", PyString_AS_STRING(self->operator));
+		snprintf(spec, sizeof(spec), "OO|O:%s", PyString_AS_STRING(self->oprator));
 		if (!PyArg_ParseTuple(args, spec, &in1, &in2, &out))
 			return NULL;
 		return _cached_dispatch2((PyObject *) self, in1, in2, out);
@@ -685,29 +680,31 @@ _accumulate_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *oty
 		{
 			return PyErr_Format( 
 				PyExc_ValueError,
-		  "Supplied output array does not have the appropriate shape");
+				"Supplied output array does not have the appropriate shape");
 		}
 		toutarr = outarr;
 		Py_INCREF(toutarr);
 	}
-
 	assert(NA_NumArrayCheck(inarr));
 	if (NA_elements((PyArrayObject *)inarr)) {
 		fc_in = _firstcol(inarr);
-		if (inarr != toutarr)
+		if (inarr != toutarr) {
 			fc_out = _firstcol(toutarr);
-		
-		result = PyObject_CallMethod(toutarr, "_copyFrom", "(O)", inarr);
-		_firstcol_undo(inarr, &fc_in);
-		if (inarr != toutarr)
+			result = PyObject_CallMethod(toutarr, "_copyFrom", "(O)", inarr);
+			_firstcol_undo(inarr, &fc_in);
 			_firstcol_undo(toutarr, &fc_out);
+		} else {
+			result = PyObject_CallMethod(toutarr, "_copyFrom", "(O)", inarr);
+			_firstcol_undo(inarr, &fc_in);
+		}
 		if (!result) {
 			Py_DECREF(toutarr);
 			return NULL;
 		} else {
 			Py_DECREF(result); /* None */
 		}
-	} else {} /* 0-element case requires nothing */
+	} else {
+	} /* 0-element case requires nothing */
 	return toutarr;
 }
 
@@ -772,11 +769,11 @@ _reduce_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *otype)
 		for(i=0; i<inarra->nd; i++)
 			toutarra->dimensions[i] = inarra->dimensions[i];
 /*		if (toutarr->nd < toutarr->nstrides) {
-			diff = toutarr->nstrides - toutarr->nd;
-			for(i=0; i<toutarr->nd; i++)
-				toutarr->strides[i] = toutarr->strides[i+diff];
-			toutarr->nstrides = toutarr->nd;
-		}
+  diff = toutarr->nstrides - toutarr->nd;
+  for(i=0; i<toutarr->nd; i++)
+  toutarr->strides[i] = toutarr->strides[i+diff];
+  toutarr->nstrides = toutarr->nd;
+  }
 */
 		NA_updateStatus(toutarra);
 	}
@@ -815,14 +812,14 @@ _cum_lookup(PyObject *self, char *cumop, PyObject *in1, PyObject *out,
 		if (!strcmp(cumop, "R")) {
 #if _PYTHON_CALLBACKS
 			wout = PyObject_CallMethod(
-			   self, "_reduce_out", "(OOO)", in1, out, otype);
+				self, "_reduce_out", "(OOO)", in1, out, otype);
 #else
 			wout = _reduce_out(self, in1, out, otype);
 #endif
 		} else {
 #if _PYTHON_CALLBACKS
 			wout = PyObject_CallMethod(
-			   self, "_accumulate_out", "(OOO)", in1, out, otype);
+				self, "_accumulate_out", "(OOO)", in1, out, otype);
 #else
 			wout = _accumulate_out(self, in1, out, otype);
 #endif
@@ -846,7 +843,7 @@ _Py_cum_lookup(PyObject *self, PyObject *args)
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_lookup: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_lookup only works on BinaryUFuncs.");
 	}
@@ -876,7 +873,7 @@ _Py_cum_fast_exec(PyObject *self, PyObject *args)
 	if (!PyArg_ParseTuple(args, "OOO:_cum_fast_exec", &in1, &out, &cached))
 		return NULL;
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, 
 			"_cum_fast_exec only works on BinaryUFuncs.");
@@ -972,7 +969,7 @@ _Py_cum_slow_exec(PyObject *self, PyObject *args)
 	PyObject *in1, *out, *cached;
 	if (!PyArg_ParseTuple(args, "OOO:_cum_slow_exec", &in1, &out, &cached))
 		return NULL;
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, 
 			"_cum_slow_exec only works on BinaryUFuncs.");
@@ -1046,7 +1043,7 @@ _Py_cum_exec(PyObject *self, PyObject *args)
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_exec: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_exec only works on BinaryUFuncs.");
 	}
@@ -1154,7 +1151,7 @@ _Py_cum_cached(PyObject *self, PyObject *args)
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_cached: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_cached only works on BinaryUFuncs.");
 	}
@@ -1168,12 +1165,12 @@ _moveToLast(int dim, int nshape, maybelong *shape)
 	int i, j;
 	maybelong temp[MAXDIM];
 	for(i=j=0; i<nshape; i++)
-	  if  (i != dim)
-	    temp[j++] = shape[i];
-	  else
-	    temp[nshape-1] = shape[i];
+		if  (i != dim)
+			temp[j++] = shape[i];
+		else
+			temp[nshape-1] = shape[i];
 	for(i=0; i<nshape; i++)
-	  shape[i] = temp[i];
+		shape[i] = temp[i];
 }
 
 static PyObject *
@@ -1199,7 +1196,7 @@ _cum_swapped(PyObject *self, PyObject *in1, int dim,
 	
 #if _PYTHON_CALLBACKS
 	_out = PyObject_CallMethod( (PyObject *) self, "_cum_cached", "sOOO", 
-				     cumop, in1, out, type);
+				    cumop, in1, out, type);
 #else
 	_out = _cum_cached(self, cumop, in1, out, type);
 #endif
@@ -1254,7 +1251,7 @@ _Py_cum_swapped(PyObject *self, PyObject *args)
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_swapped: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_swapped only works on BinaryUFuncs.");
 	}
@@ -1302,7 +1299,7 @@ _Py_accumulate(PyObject *self, PyObject *args, PyObject *keywds)
 		return PyErr_Format(
 			PyExc_TypeError, "accumulate: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "_cum_swapped only works on BinaryUFuncs.");
 	}
@@ -1334,7 +1331,7 @@ _Py_areduce(PyObject *self, PyObject *args, PyObject *keywds)
 		return PyErr_Format(
 			PyExc_TypeError, "reduce: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "areduce: only works on BinaryUFuncs.");
 	}
@@ -1366,7 +1363,7 @@ _Py_reduce(PyObject *self, PyObject *args, PyObject *keywds)
 		return PyErr_Format(
 			PyExc_TypeError, "reduce: out must be a NumArray");
 
-	if (((PyUfuncObject*)self)->arity != 2) {
+	if (((PyUfuncObject*)self)->n_inputs != 2) {
 		return PyErr_Format(
 			PyExc_TypeError, "reduce only works on BinaryUFuncs.");
 	}
@@ -1604,7 +1601,7 @@ _callOverDimensions(PyObject *objects, PyObject *outshape,
 		shape0 = PyTuple_GET_ITEM(shape, 0) ;
 		if (!PyInt_Check(shape0) && !PyLong_Check(shape0))
 			return PyErr_Format(PyExc_TypeError,
-				    "_callOverDimensions: shape0 is not an int.");
+					    "_callOverDimensions: shape0 is not an int.");
 	}
 	nregShapeIters = PyTuple_GET_ITEM(blocking, 0);
 	if (!PyInt_Check(nregShapeIters))
@@ -1636,7 +1633,7 @@ _Py_callOverDimensions(PyObject *m, PyObject *args)
 		PyObject *obj = PyTuple_GET_ITEM(objects, i);
 		if (!obj && !NA_ConverterCheck(obj) && !NA_OperatorCheck(obj))
 			return PyErr_Format(PyExc_TypeError, 
-			     "_callOverDimensions: bad converter or operator");
+					    "_callOverDimensions: bad converter or operator");
 	}
 	if (!PyTuple_Check(outshape) || PyTuple_GET_SIZE(outshape)<level)
 		return PyErr_Format(PyExc_ValueError, 
@@ -1727,16 +1724,16 @@ _slow_exec1(PyObject *ufunc, PyObject *in1, PyObject *out, PyObject *cached)
 		return NULL;
 	if (!PyTuple_Check(ufargs) || PyTuple_GET_SIZE(ufargs) != 3)
 		return PyErr_Format(PyExc_ValueError,
-			"_slow_exec1: problem with ufargs tuple.");
+				    "_slow_exec1: problem with ufargs tuple.");
 	if (!PyArg_ParseTuple(ufargs, "OOi:_slow_exec1 ufargs", 
 			      &inputs, &outputs, &maxitemsize))
 		return NULL;
 	if (!PyTuple_Check(inputs) || PyTuple_GET_SIZE(inputs) != 1)
 		return PyErr_Format(PyExc_ValueError, 
-			"_slow_exec1: problem with inputs tuple.");
+				    "_slow_exec1: problem with inputs tuple.");
 	if (!PyTuple_Check(outputs) || PyTuple_GET_SIZE(outputs) != 1)
 		return PyErr_Format(PyExc_ValueError, 
-			"_slow_exec1: problem with outputs tuple.");
+				    "_slow_exec1: problem with outputs tuple.");
 	if (maxitemsize <= 0)
 		return PyErr_Format(PyExc_ValueError,
 				    "_slow_exec1: maxitemsize <= 0");
@@ -1800,7 +1797,7 @@ _Py_slow_exec1(PyObject *module, PyObject *args)
 
 	if (!PyTuple_Check(cached) || PyTuple_GET_SIZE(cached) < 4)
 		return PyErr_Format(PyExc_ValueError,
-			    "_Py_slow_exec1: problem with cache tuple.");
+				    "_Py_slow_exec1: problem with cache tuple.");
 
 	return _slow_exec1(ufunc, in1, out, cached);
 }
@@ -1840,7 +1837,7 @@ _Py_cache_exec1(PyObject *module, PyObject *args)
 {
 	PyObject *ufunc, *in1, *out, *cached;
 	if (!PyArg_ParseTuple(args, "OOOO:_Py_cache_exec1", 
-		      &ufunc, &in1, &out, &cached))
+			      &ufunc, &in1, &out, &cached))
 		return NULL;
 
 	if (!NA_NumArrayCheck(out))
@@ -1856,8 +1853,8 @@ _Py_cache_exec1(PyObject *module, PyObject *args)
 
 static PyObject *
 _cache_lookup1(PyObject *ufunc, PyObject *in1, PyObject *out,
-		PyObject **oin1, PyObject **oout,
-		PyObject **ocache)
+	       PyObject **oin1, PyObject **oout,
+	       PyObject **ocache)
 {
 	PyObject *ctuple;
 
@@ -1872,7 +1869,7 @@ _cache_lookup1(PyObject *ufunc, PyObject *in1, PyObject *out,
 		    (PyTuple_GET_SIZE(ctuple) != 6)) {
 			Py_DECREF(ctuple);
 			return PyErr_Format(PyExc_RuntimeError,
-				  "_cache_lookup1: invalid cache tuple");
+					    "_cache_lookup1: invalid cache tuple");
 		}
 		cin1 = PyTuple_GET_ITEM(ctuple, 5);
 		cin1 = _restuff_pseudo(cin1, in1);
@@ -1888,11 +1885,11 @@ _cache_lookup1(PyObject *ufunc, PyObject *in1, PyObject *out,
 		} else { /* output array was supplied */
 			if (!NA_NumArrayCheck(out))
 				return PyErr_Format(PyExc_TypeError,
-					 "_cache_lookup1: output array was not a numarray");
+						    "_cache_lookup1: output array was not a numarray");
 			if  (!NA_ShapeEqual((PyArrayObject *)cin1, 
 					    (PyArrayObject *)out))
 				return PyErr_Format(PyExc_ValueError,
-				    "Supplied output array does not have appropriate shape");
+						    "Supplied output array does not have appropriate shape");
 			Py_INCREF(out);
 		}
 		*oin1 = cin1;
@@ -1902,7 +1899,7 @@ _cache_lookup1(PyObject *ufunc, PyObject *in1, PyObject *out,
 	} else { /* cache miss */
 		PyObject *result;
 		result = PyObject_CallMethod(ufunc, 
-			     "_cache_miss1", "(OO)", in1, out);
+					     "_cache_miss1", "(OO)", in1, out);
 		if (!result) return NULL;
 		if (!PyArg_ParseTuple(result, "OOO:_cache_lookup1",
 				      oin1, oout, ocache))
@@ -1925,7 +1922,7 @@ _Py_cache_lookup1(PyObject *module, PyObject *args)
 	
 	if (!PyArg_ParseTuple(args, "OOO:_Py_cache_lookup1", 
 			      &ufunc, &in1, &out))
-	return NULL;
+		return NULL;
 
 	result = _cache_lookup1(ufunc, in1, out, 
 				&oin1, &oout, &ocached);
@@ -2119,7 +2116,7 @@ _Py_slow_exec2(PyObject *module, PyObject *args)
 
 	if (!PyTuple_Check(cached) || PyTuple_GET_SIZE(cached) != 7)
 		return PyErr_Format(PyExc_ValueError,
-			    "_Py_slow_exec2: problem with cache tuple.");
+				    "_Py_slow_exec2: problem with cache tuple.");
 
 	return _slow_exec2(ufunc, in1, in2, out, cached);
 }
@@ -2177,8 +2174,8 @@ _Py_cache_exec2(PyObject *module, PyObject *args)
 
 static PyObject *
 _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
-		PyObject **oin1, PyObject **oin2, PyObject **oout,
-		PyObject **ocache)
+	       PyObject **oin1, PyObject **oin2, PyObject **oout,
+	       PyObject **ocache)
 {
 	PyObject *ctuple, *result=0, *bcast=0;
 
@@ -2191,12 +2188,12 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 		if (!bcast) return NULL;
 		if (!PyTuple_Check(bcast) || (PyTuple_GET_SIZE(bcast) != 2))
 			return PyErr_Format(PyExc_RuntimeError,
-			    "_cache_lookup2: invalid broadcast result.");
+					    "_cache_lookup2: invalid broadcast result.");
 		in1 = PyTuple_GET_ITEM(bcast, 0);
 		in2 = PyTuple_GET_ITEM(bcast, 1);
 		if (!NA_NumArrayCheck(in1) || !NA_NumArrayCheck(in2))
 			return PyErr_Format(PyExc_RuntimeError,
-			    "_cache_lookup2: invalid broadcast results.");
+					    "_cache_lookup2: invalid broadcast results.");
 	}
 
 	tc_stop_clock("_cache_lookup2 broadcast");
@@ -2251,7 +2248,7 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 			PyObject *compare_to;
 			if (!NA_NumArrayCheck(out))
 				return PyErr_Format(PyExc_TypeError,
-					 "_cache_lookup2: output array was not a numarray");
+						    "_cache_lookup2: output array was not a numarray");
 			if (!strcmp(insig_str, "vv") ||
 			    !strcmp(insig_str, "vs")) {
 				compare_to = cin1;
@@ -2260,13 +2257,13 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 			} else {
 				if (((PyArrayObject *)out)->nd)
 					return PyErr_Format(PyExc_ValueError,
-					    "_cache_lookup2: scalar output array rank > 0.");
+							    "_cache_lookup2: scalar output array rank > 0.");
 				compare_to = out;
 			}			
 			if  (!NA_ShapeEqual((PyArrayObject *)out, 
 					    (PyArrayObject *)compare_to))
 				return PyErr_Format(PyExc_ValueError,
-				    "Supplied output array does not have appropriate shape");
+						    "Supplied output array does not have appropriate shape");
 			Py_INCREF(out);
 		}
 		*oin1 = cin1;
@@ -2278,7 +2275,7 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 	} else { /* cache miss */
 		tc_start_clock("_cache_lookup2 miss");
 		result = PyObject_CallMethod(ufunc, 
-			     "_cache_miss2", "(OOO)", in1, in2, out);
+					     "_cache_miss2", "(OOO)", in1, in2, out);
 		if (!result) goto _fail;
 		if (!PyArg_ParseTuple(result, "OOOO:_cache_lookup2", 
 				      oin1, oin2, oout, ocache))
@@ -2326,7 +2323,7 @@ _cached_dispatch2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out)
 	tc_start_clock("cached_dispatch2 overall");
 	tc_start_clock("cached_dispatch2 lookup");
 	result = _cache_lookup2( ufunc, in1, in2, out,
-			&oin1, &oin2, &oout, &ocached);
+				 &oin1, &oin2, &oout, &ocached);
 	tc_stop_clock("cached_dispatch2 lookup");
 	if (!result) 
 		return NULL;
@@ -2519,11 +2516,10 @@ static PyMethodDef _ufunc_functions[] = {
 
 
 DL_EXPORT(void)
-init_ufunc(void)
+	init_ufunc(void)
 {
 	PyObject *m;
 
-	_ufunc_type.tp_base = NULL;
 	_ufunc_type.tp_alloc = PyType_GenericAlloc;
 
 	if (PyType_Ready(&_ufunc_type) < 0)
