@@ -1,6 +1,7 @@
 #include <Python.h>
 #include <stdio.h>
 #include "arraybase.h"   /* For Int64 */
+#include "nummacro.h"
 
 #if defined(_MSC_VER)
 #define SIZE_MAX 0x7CFFFFFFL
@@ -10,7 +11,7 @@
 #include <limits.h>
 #define SIZE_MAX ULONG_MAX
 #else
-#include <stdint.h>
+#include <inttypes.h>
 #endif
 
 #ifndef SIZE_MAX
@@ -23,8 +24,8 @@ static PyObject *memoryError;
 
 typedef struct {
   PyObject_HEAD
-  unsigned char     *ptr;
-  unsigned char     *base;
+  char     *ptr;
+  char     *base;
   Int64     size;
   PyObject *master;
 } MemoryObject;
@@ -46,7 +47,7 @@ _new_memory(Int64 size)
 	if (!(memory = PyObject_New(MemoryObject, &MemoryType))) 
 		return NULL;
 
-	memory->base = (unsigned char *) PyMem_New(
+	memory->base = (char *) PyMem_New(
 		double, size/sizeof(double) + (size%sizeof(double)!=0) + 1);
 	if (!memory->base) {
 		PyErr_Format(PyExc_MemoryError, "Couldn't allocate requested memory");
@@ -54,7 +55,7 @@ _new_memory(Int64 size)
 	}
 	base = ((unsigned long) memory->base) / sizeof(double);
 	align = (((unsigned long) memory->base) % sizeof(double)) != 0;
-	memory->ptr = (unsigned char *) ((base+align) * sizeof(double));
+	memory->ptr = (char *) ((base+align) * sizeof(double));
 	memory->size = size;
 	memory->master = NULL;
 	return (PyObject *) memory;
@@ -88,7 +89,7 @@ memory_buffer(PyObject *self, PyObject *args)  /* deprecated */
 }
 
 static PyObject *
-memory_alias(PyObject *master, unsigned char *ptr, int size)
+memory_alias(PyObject *master, char *ptr, int size)
 {
 	MemoryObject *memory;
 	if (size < 0)
@@ -355,7 +356,7 @@ memory_tolist(MemoryObject *self, PyObject  *args)
 	l = PyList_New(self->size);
 	if (!l) return NULL;
 	for(i=0; i<self->size; i++) {
-		PyObject *o = PyInt_FromLong(self->ptr[i]);
+		PyObject *o = PyInt_FromLong(((unsigned char *)self->ptr)[i]);
 		if (!o) { 
 			Py_DECREF(l);
 			return NULL;
@@ -389,7 +390,7 @@ memory_getattr(PyObject *obj, char *name)
 static PyTypeObject MemoryType = {
     PyObject_HEAD_INIT(NULL)
     0,
-    "Memory",
+    "numarray.memory.Memory",
     sizeof(MemoryObject),
     1,                           /* per item cost */
     memory_dealloc,              /* tp_dealloc */
@@ -435,7 +436,8 @@ initmemory(void)
 	MemoryType.ob_type = &PyType_Type;
 	m = Py_InitModule("memory", module_methods);
 	d = PyModule_GetDict(m);
-	memoryError = PyErr_NewException("memory.error", NULL, NULL);
+	memoryError = PyErr_NewException("numarray.memory.error", NULL, NULL);
 	PyDict_SetItemString(d, "error", memoryError);
 	PyDict_SetItemString(d, "MemoryType", (PyObject *) &MemoryType);
+	ADD_VERSION(m);
 }
